import { Book, BookDetailedSchema } from "@/models/book";

// We do not implement Book or ModifyBook and instead only implement DetailedBook for homogeneity

export class DetailedBook {
	id: number;
	title: string;
	isbn: string | null;
	category: string[];
	authors: string | null;
	subtitle: string | null;
	// Optional fields ↓
	last_updated?: Date;
	deleted?: boolean;
	lccn?: string;
	edition?: string;
	publisher?: string;
	publish_year?: number;
	publish_month?: string;
	publish_location?: string;
	pages?: number;
	weight?: string;

	constructor(book: Book) {
		this.id = book.id;
		this.title = book.title;
		this.category = book.category;
		this.subtitle = book.subtitle;
		this.authors = book.authors;
		this.isbn = book.isbn;
		// Optional fields ↓
		this.deleted = book.deleted;
		this.last_updated = book.last_updated;
		this.lccn = book.lccn;
		this.edition = book.edition;
		this.publisher = book.publisher;
		this.publish_year = book.publish_year;
		this.publish_month = book.publish_month;
		this.publish_location = book.publish_location;
		this.pages = book.pages;
		this.weight = book.weight;

		// Validate the book object
		const result = BookDetailedSchema.safeParse(book);
		if (!result.success) {
			let error = "";
			result.error.errors.forEach((err) => {
				error += err.message;
			});
			throw new Error(error);
		}
	}

	// Getters
	get authorsList() {
		return this.authors?.split(";")?.map((v) => v.trim()) || [];
	}

	// Setters
	set authorsList(authors: string[]) {
		this.authors = authors.join("; ");
	}
}
